#!/usr/bin/env python
## Version 1.10

import sys, time, os, operator
import Daemon
import shutil
import logging
import grp
import datetime
import ftplib
import socket
import smtplib
from stat import *
from email.MIMEMultipart import MIMEMultipart
from email.MIMEBase import MIMEBase
from email.MIMEText import MIMEText
from email.Utils import COMMASPACE, formatdate
from email import Encoders

class NMCFiledistDaemon(Daemon.Daemon):
	def run(self):

		sys.stderr.write("Executing Run \n")
		ISDMDir = '/pub/gtspp'
		NMCOutDir = AMVERWorkDir+'NOSSRVR/nmc/'
		AT= time.gmtime(time.time())
		ARCHFile = AMVERWorkDir+"NOSSRVR/logfiles/ISDMDistrib/ISDM-"+("%04d" %AT[0])+("%02d" %AT[1])+("%02d" %AT[2])+".log"
		ARCHLog=logging.getLogger("ArchLog")
		ARCHLog.setLevel(logging.DEBUG)
		handler = logging.FileHandler(ARCHFile)
		Form = logging.Formatter('%(asctime)s %(levelname)-8s %(module)s %(lineno)d %(message)s','%a, %d %b %Y %H:%M:%S')
		handler.setFormatter(Form)
		ARCHLog.addHandler(handler)
		os.chmod(ARCHFile,0660)  #Change mode on logfile
		
		ARCHLog.debug('AMVERWorkDir = %s' %AMVERWorkDir)
		ARCHLog.debug('ISDMftp = %s' %ISDMftp)
		ARCHLog.debug('DeliverNMCFiles = %s' %DeliverNMCFiles)
		
		DeliverBUFR = False
		DeliverISDM = False

		if(DeliverNMCFiles=='true'):
			DeliverNMC = True
			
		
			

		times=time.time()
		while True:
			Elapsed = int((time.time()-times)/60.0)				
			ARCHLog.info('Active for %s minutes\r\n' %str(Elapsed))
			time.sleep(60)  # wait one minute before checking for new messages
			
			st = os.stat(pidfile) # Touch the pid file so we know we are still running
			atime = st[ST_ATIME]
			FT = time.time()
			os.utime(pidfile,(atime,FT))
			
## Check the date and time to see if a new log file shoule be created.
			AT = time.gmtime(time.time())
			NewARCHFile = AMVERWorkDir+"NOSSRVR/logfiles/ISDMDistrib/ISDM-"+("%04d" %AT[0])+("%02d" %AT[1])+("%02d" %AT[2])+".log"
			if os.path.exists(NewARCHFile):
				pass
			else:
				ARCHLog.removeHandler(handler)
				handler.close()
				handler=logging.FileHandler(NewARCHFile)
				handler.setFormatter(Form)
				ARCHLog.addHandler(handler)
				os.chmod(NewARCHFile,0660)  #Change mode on logfile
				ARCHLog.debug('AMVERWorkDir = %s' %AMVERWorkDir)
				ARCHLog.debug('ISDMftp = %s' %ISDMftp)
				ARCHLog.debug('DeliverNMCFiles = %s' %DeliverNMCFiles)


			Files2Send=0
			Dir = os.listdir(NMCOutDir)
			DST = os.stat(NMCOutDir)
			datime = DST[ST_ATIME]
			SourceDir = NMCOutDir
			for N in Dir:
				FileName = SourceDir+N
				if os.path.splitext(FileName)[1].upper().find('.NMC')<0:
					continue
				if(not os.path.isdir(FileName)):
					Files2Send = Files2Send+1

			if(Files2Send==0):
				continue     ## No files to send so no use connecting
				
			if(not DeliverNMC):  # Do not deliver NDC Files
				for N in Dir:
					FileName = SourceDir+N
					if(os.path.isdir(FileName)):  # Check to see if the file is a directory
						continue
					else:
						shutil.move(FileName,NMCOutDir+'archive/'+N) 
				continue

			
			
#			try:
#				ftp = ftplib.FTP()
#				ftp.connect(ISDMftp)
#				ftp.set_pasv(False)
#				#      ftp = ftplib.FTP('seas.amverseas.noaa.gov')
#				ftp.set_debuglevel(2)
#
#				ftp.login('NOAA','n0@@456')
#				ftp.cwd(ISDMDir)
#			except ftplib.all_errors, e:
#				ARCHLog.info("FTP ERROR %s \r\n" % str(e))
#				if(e.args[0] not in [10054,11001,111,110]):
#				try:
#					ftp.quit()
#				except:
#					ftp.close() ##ftp.quit()
#				continue
#			except socket.error, e:
#				ARCHLog.info("Socket Error %s %s" %(e.errno,e.strerror))
#				try:
#					ftp.quit()
#				except:
#					ftp.close() ##ftp.quit()
#				continue
#			Files2Send =0

			for N in Dir:

				if os.path.isdir(NMCOutDir+N):
					continue
				else:
					SourceDir =NMCOutDir
					FileName = NMCOutDir+N
					if os.path.splitext(FileName)[1].upper().find('.NMC')<0:
						continue
					elif datime - os.stat(FileName)[ST_MTIME] <30:
						continue
					else:
						msg = MIMEMultipart()
						send_from = 'NOAA_Mailerd@as101.amverseas.noaa.gov'
						msg['From'] = send_from
						msg['To']='meds-sdmm@dfo-mpo.gc.ca'
#						msg['To']='paul.chinn@noaa.gov'
						msg['Subject'] = 'NMC Files Attached'

						msg.attach( MIMEText('Attached are *.nmc files from NOAA') )

						part = MIMEBase('application', "octet-stream")
						part.set_payload( open(FileName,"rb").read() )
						Encoders.encode_base64(part)
						part.add_header('Content-Disposition', 'attachment; filename="%s"' % os.path.basename(N))
						msg.attach(part)
		#				print os.path.basename(N)
			        	send_to =('meds-sdmm@dfo-mpo.gc.ca',)
#			        	send_to =('paul.chinn@noaa.gov',)
			        	smtp1 = smtplib.SMTP('localhost')
			        	smtp1.sendmail(send_from, send_to, msg.as_string())
			        	smtp1.close()
			        	if Files2Send >1:
				        	time.sleep(30)
				        	
					shutil.move(FileName,NMCOutDir+'archive/'+N) 
					ARCHLog.info(FileName)
					ARCHLog.info(NMCOutDir+'archive/'+N)

##		except Exception,e:
##			ARCHLog.exception(str(e))
##			sys.exit(2)
				
	
	
def ReadConf(ConfigFile):
##	global CODE41Archive, CODE41Inbox, CODE43Inbox,  AMVERWorkDir, NWSMessOutDir, CODE43Archive
	global AMVERWorkDir, ISDMftp,  DeliverBUFRFiles, DeliverNMCFiles
	if ConfigFile != '':
		try:
			nosdist = open(ConfigFile,"r")
		except Exception, e:
			print str(e)
			sys.exit(2)
		
#		print "configfile is \n", ConfigFile
		InfoCount=0
		while 1:
			line=nosdist.readline()
#			print line
			if len(line)==0:
				break
			colon = line.find(':')
#			print line[0:colon]
			if line[0:colon]=="ISDMftp":
				ISDMftp=line[colon+1:].strip()
			elif line[0:colon]=="AMVERWorkDir":
				AMVERWorkDir=line[colon+1:].strip()
			elif line[0:colon]=="DeliverNMCFiles":
				DeliverNMCFiles=line[colon+1:].strip()
		nosdist.close() #close the config file
		return True
	else:
		return False

if __name__ == "__main__":
	logfile = 'ISDMdist'
	ISDMftp = "ftp.aoml.noaa.gov"
	AMVERWorkDir=''
	ConfigFile=''
	pidfile = 'ISDMFiledist.pid'
	pidfileDir = '/home/AMVER/'
	version = 1.0
	
#  Change the user and group to amver and AMVER

	group = grp.getgrnam("AMVER")
#	print group
	os.setregid(group[2],group[2])
	os.setreuid(group[2],group[2])

#	sys.stderr.write(str(sys.argv) + str(len(sys.argv))+'\n')
	if len(sys.argv)>=2 and operator.mod(len(sys.argv),2)==0:
		command = sys.argv[1]
		for i in range(len(sys.argv)):
#			print ('%d  %s \n' % (i,sys.argv[i]))
			if i<2:
				pass
			elif operator.mod(i,2)==0:
				if sys.argv[i]== '-P':
					pidfile=sys.argv[i+1]
				elif sys.argv[i]=='-L':
					logfile=sys.argv[i+1]
				elif sys.argv[i]=='-C':
					ConfigFile=sys.argv[i+1]
					if ReadConf(ConfigFile):
						print AMVERWorkDir, ISDMftp,  DeliverNMCFiles
						pidfileDir = AMVERWorkDir
						pass
					else:
						print " Failed to retrieve information from config file"
						sys.exit(2)
			else:
				pass
		pidfile = (pidfileDir+pidfile)
		daemon = NMCFiledistDaemon(pidfile)
		if 'start' == command:
			print "starting with pidfile = %s\r\n" %pidfile
			daemon.start()
#		else:
#			print "usage: %s start|stop|restart -P pidfile -L logfile -C ConfigFile" % sys.argv[0]
#			sys.exit(2)
#			sys.exit(0)
		elif 'stop' == command:
			daemon.stop()
		elif 'restart' == command:
			daemon.restart()
		elif sys.argv[i]=='-V':
			print 'version %6.2f' %version
		else:
			print "Unknown command"
			sys.exit(2)
	else:
		print "usage: %s start|stop|restart -P pidfile -L logfile -C ConfigFile Directory" % sys.argv[0]
		sys.exit(2)
