#!/usr/bin/env python
## Version 1.10
## Jan 6, 2013.  Modified to make sure file has been in the directory at least 30 seconds before trying to transmit to the Lab


import sys, time, os, operator
import Daemon
import shutil
import logging
import grp
import datetime
import ftplib
import socket
from stat import *
class NOSBUFRdistDaemon(Daemon.Daemon):
	def run(self):

		sys.stderr.write("Executing Run \n")
		AOMLBUFRDir = 'phod/incoming/xbt/seas/bufr'
		BUFROutDir = AMVERWorkDir+'NOSSRVR/bufr_queue/'
		AT= time.gmtime(time.time())
		ARCHFile = AMVERWorkDir+"NOSSRVR/logfiles/NOSDistrib/NOS-"+("%04d" %AT[0])+("%02d" %AT[1])+("%02d" %AT[2])+".log"
		ARCHLog=logging.getLogger("ArchLog")
		ARCHLog.setLevel(logging.DEBUG)
		handler = logging.FileHandler(ARCHFile)
		Form = logging.Formatter('%(asctime)s %(levelname)-8s %(module)s %(lineno)d %(message)s','%a, %d %b %Y %H:%M:%S')
		handler.setFormatter(Form)
		ARCHLog.addHandler(handler)
		os.chmod(ARCHFile,0660)  #Change mode on logfile
		
		ARCHLog.debug('AMVERWorkDir = %s' %AMVERWorkDir)
		ARCHLog.debug('AOMLftp = %s' %AOMLftp)
		ARCHLog.debug('DeliverBUFRFiles = %s' %DeliverBUFRFiles)
		
		DeliverBUFR = False
		DeliverISDM = False

		if(DeliverBUFRFiles=='true'):
			DeliverBUFR = True
			
		
			

		times=time.time()
		while True:
			Elapsed = int((time.time()-times)/60.0)				
			ARCHLog.info('Active for %s minutes\r\n' %str(Elapsed))
			time.sleep(60)  # wait one minute before checking for new messages
			
			st = os.stat(pidfile) # Touch the pid file so we know we are still running
			atime = st[ST_ATIME]
			FT = time.time()
			os.utime(pidfile,(atime,FT))
			
## Check the date and time to see if a new log file shoule be created.
			AT = time.gmtime(time.time())
			NewARCHFile = AMVERWorkDir+"NOSSRVR/logfiles/NOSDistrib/NOS-"+("%04d" %AT[0])+("%02d" %AT[1])+("%02d" %AT[2])+".log"
			if os.path.exists(NewARCHFile):
				pass
			else:
				ARCHLog.removeHandler(handler)
				handler.close()
				handler=logging.FileHandler(NewARCHFile)
				handler.setFormatter(Form)
				ARCHLog.addHandler(handler)
				os.chmod(NewARCHFile,0660)  #Change mode on logfile
				ARCHLog.debug('AMVERWorkDir = %s' %AMVERWorkDir)
				ARCHLog.debug('AOMLftp = %s' %AOMLftp)
				ARCHLog.debug('DeliverBUFRFiles = %s' %DeliverBUFRFiles)


			Files2Send=0
			Dir = os.listdir(BUFROutDir)
			DST = os.stat(BUFROutDir)
			datime = DST[ST_ATIME]
			SourceDir = BUFROutDir
			for N in Dir:
				FileName = SourceDir+N
				if(not os.path.isdir(FileName)):
					if datime - os.stat(FileName)[ST_MTIME] <30:  ## at least 30 seconds
						continue
					Files2Send = Files2Send+1

			if(Files2Send==0):
				continue     ## No files to send so no use connecting
				
			if(not DeliverBUFR):  # Do not deliver NDC Files
				for N in Dir:
					FileName = SourceDir+N
					if(os.path.isdir(FileName)):  # Check to see if the file is a directory
						continue
					else:
						os.remove(FileName)   #  if not a directory, remove the file
				continue

			
			
			try:
				ftp = ftplib.FTP()
				ftp.connect(AOMLftp)
				ftp.set_pasv(False)
				#      ftp = ftplib.FTP('seas.amverseas.noaa.gov')
				ftp.set_debuglevel(1)

				ftp.login('xbt','!go2#B@y')
				ftp.cwd(AOMLBUFRDir)
			except ftplib.all_errors, e:
				ARCHLog.info("FTP ERROR %s \r\n" % str(e))
				if(e.args[0] not in [10054,11001,111,110]):
					ftp.quit()
				continue
			except socket.error, e:
				ARCHLog.info("Socket Error %s %s" %(e.errno,e.strerror))
				ftp.quit()
				continue
#			Files2Send =0

			ARCHLog.info('Sending %d files to AOML' %Files2Send)
			
			for N in Dir:

				if os.path.isdir(BUFROutDir+N):
					continue
				else:
					SourceDir =BUFROutDir
					FileName = BUFROutDir+N
					ARCHLog.info("Time Diff = %d FileName= %s" %(datime - os.stat(FileName)[ST_MTIME], N))
					if(os.path.isdir(FileName)):
						continue
					elif datime - os.stat(FileName)[ST_MTIME] <30:
						continue
					else:
						filen = open('%s'%FileName,'rb')
						try:
							ftp.storbinary('STOR '+N, filen)
							filen.seek(0)
	#						ftp.delete(FileName)
							filen.close()
							shutil.move(FileName,BUFROutDir+'archive/'+N) 
							ARCHLog.info(FileName)
							ARCHLog.info(BUFROutDir+'archive/'+N)
#Log.flush()
						except socket.timeout:
							ARCHLog.info("Timeout on Connection with file "+N+"\r\n")
							break
						except OSError, e:
							if e.errno == 13:
								ARCHLog.info("Permission denied to move file %s %s\r\n" %(e.errno,e.strerror))
							break
						except ftplib.all_errors, e:
							ARCHLog.info("FTP ERROR %s \r\n" % str(e))
#							if(e.args[0] not in [10054,11001]):
							break
			try:
				ftp.quit()
				ARCHLog.info("FTP Quit")
			except:
				ARCHLog.info("FTP Close")
				ftp.close()
			continue

##		except Exception,e:
##			ARCHLog.exception(str(e))
##			sys.exit(2)
				
	
	
def ReadConf(ConfigFile):
##	global CODE41Archive, CODE41Inbox, CODE43Inbox,  AMVERWorkDir, NWSMessOutDir, CODE43Archive
	global AMVERWorkDir, AOMLftp,  DeliverBUFRFiles, DeliverNMCFiles
	if ConfigFile != '':
		try:
			nosdist = open(ConfigFile,"r")
		except Exception, e:
			print str(e)
			sys.exit(2)
		
#		print "configfile is \n", ConfigFile
		InfoCount=0
		while 1:
			line=nosdist.readline()
#			print line
			if len(line)==0:
				break
			colon = line.find(':')
#			print line[0:colon]
			if line[0:colon]=="AOMLftp":
				AOMLftp=line[colon+1:].strip()
			elif line[0:colon]=="AMVERWorkDir":
				AMVERWorkDir=line[colon+1:].strip()
			elif line[0:colon]=="DeliverBUFRFiles":
				DeliverBUFRFiles=line[colon+1:].strip()
		nosdist.close() #close the config file
		return True
	else:
		return False

if __name__ == "__main__":
	logfile = 'NOSBUFRdist'
	AOMLftp = "ftp.aoml.noaa.gov"
	AMVERWorkDir=''
	ConfigFile=''
	pidfile = 'NOSBUFRdist.pid'
	pidfileDir = '/home/AMVER/'
	version = 1.0
	
#  Change the user and group to amver and AMVER

	group = grp.getgrnam("AMVER")
#	print group
	os.setregid(group[2],group[2])
	os.setreuid(group[2],group[2])

#	sys.stderr.write(str(sys.argv) + str(len(sys.argv))+'\n')
	if len(sys.argv)>=2 and operator.mod(len(sys.argv),2)==0:
		command = sys.argv[1]
		for i in range(len(sys.argv)):
#			print ('%d  %s \n' % (i,sys.argv[i]))
			if i<2:
				pass
			elif operator.mod(i,2)==0:
				if sys.argv[i]== '-P':
					pidfile=sys.argv[i+1]
				elif sys.argv[i]=='-L':
					logfile=sys.argv[i+1]
				elif sys.argv[i]=='-C':
					ConfigFile=sys.argv[i+1]
					if ReadConf(ConfigFile):
						print AMVERWorkDir, AOMLftp,  DeliverBUFRFiles
						pidfileDir = AMVERWorkDir
						pass
					else:
						print " Failed to retrieve information from config file"
						sys.exit(2)
			else:
				pass
		pidfile = (pidfileDir+pidfile)
		daemon = NOSBUFRdistDaemon(pidfile)
		if 'start' == command:
			print "starting with pidfile = %s\r\n" %pidfile
			daemon.start()
#		else:
#			print "usage: %s start|stop|restart -P pidfile -L logfile -C ConfigFile" % sys.argv[0]
#			sys.exit(2)
#			sys.exit(0)
		elif 'stop' == command:
			daemon.stop()
		elif 'restart' == command:
			daemon.restart()
		elif sys.argv[i]=='-V':
			print 'version %6.2f' %version
		else:
			print "Unknown command"
			sys.exit(2)
	else:
		print "usage: %s start|stop|restart -P pidfile -L logfile -C ConfigFile Directory" % sys.argv[0]
		sys.exit(2)
